<?php

namespace App\Factory;

use Doctrine\DBAL\Connection;

class Report {
    private $connection;

    public function __construct(Connection $connection) {
        $this->connection = $connection;
    }

    public function orderProduction($params) {
      $sql = "
        SELECT
          S.NMSETOR AS SETOR,
          P.CDPRODUTO || ' - ' ||  P.NMPRODUTO AS PRODUTO,
          C.NRCOMANDA AS PEDIDO,
          C.IDORGCMDVENDA,
          C.DSENDECONSCOMAN AS ENDERECO,
          NVL(TRIM(IT.TXPRODCOMVEN), '') AS OBSERVACAO,
          NVL(TRIM(C.DSOBSCOMANDA), '') AS OBSERVACAOCMD,
          NVL(TO_CHAR(C.DTHRAGENDADA, 'DD/MM/YYYY'), TO_CHAR(V.DTHRABERMESA, 'DD/MM/YYYY')) AS DATA,
          NVL(TO_CHAR(C.DTHRAGENDADA, 'HH24:MI:SS'), TO_CHAR(V.DTHRABERMESA, 'HH24:MI:SS')) AS HORARIO,
          IT.QTPRODCOMVEN AS QUANTIDADE,
          CL.NMRAZSOCCLIE AS CLIENTE
        FROM VENDAREST V

        INNER JOIN COMANDAVEN C
          ON C.CDFILIAL = V.CDFILIAL
          AND C.NRVENDAREST = V.NRVENDAREST

        INNER JOIN ITCOMANDAVEN IT
          ON IT.NRVENDAREST = V.NRVENDAREST
          AND IT.CDFILIAL = V.CDFILIAL
          AND IT.NRCOMANDA = C.NRCOMANDA

        INNER JOIN CLIENTE CL
          ON CL.CDCLIENTE = V.CDCLIENTE

        INNER JOIN PRODUTO P
          ON P.CDPRODUTO = IT.CDPRODUTO

        LEFT JOIN SETORFILIAL SF
          ON SF.CDFILIAL = V.CDFILIAL

        LEFT JOIN SETORPRODUTO SP
          ON SP.CDSETOR = SF.CDSETOR
          AND SP.CDPRODUTO = IT.CDPRODUTO

        LEFT JOIN SETOR S
          ON S.CDSETOR = SF.CDSETOR

        WHERE
          V.CDFILIAL = :CDFILIAL
          AND SF.CDSETOR = :CDSETOR
          AND (
          (TRUNC(V.DTHRABERMESA)BETWEEN TO_DATE(:DTINIAGENDADA, 'DD/MM/YYYY') AND TO_DATE(:DTFIMAGENDADA, 'DD/MM/YYYY')) OR
          (TRUNC(C.DTHRAGENDADA)BETWEEN TO_DATE(:DTINIAGENDADA, 'DD/MM/YYYY') AND TO_DATE(:DTFIMAGENDADA, 'DD/MM/YYYY')))
        ORDER BY V.DTHRABERMESA DESC
      ";

      $stmt = $this->connection->prepare($sql);
      $stmt->bindValue(':CDFILIAL', $params['cdFilial']);
      $stmt->bindValue(':CDSETOR', $params['cdSetor']);
      $stmt->bindValue(':DTINIAGENDADA', $params['dtInicial']);
      $stmt->bindValue(':DTFIMAGENDADA', $params['dtFinal']);
      return $stmt->executeQuery()->fetchAllAssociative();
    }

    public function getUnit($params) {
      $search = $params['search'] ?? null;

      $sql = "
        SELECT
          CDFILIAL, NMFILIAL
        FROM FILIAL
      ";

      if (!empty($search)) {
        $sql .= "
            WHERE CDFILIAL LIKE :search
            OR UPPER(NMFILIAL) LIKE :search
        ";
      }

      $stmt = $this->connection->prepare($sql);

      if (!empty($search)) {
        $searchTerm = '%' . strtoupper($search) . '%';
        $stmt->bindValue(':search', $searchTerm);
    }

    return $stmt->executeQuery()->fetchAllAssociative();
  }

    public function getSector($params) {
      $search = $params['search'] ?? null;

      $sql = "
        SELECT
          CDSETOR, NMSETOR
        FROM SETOR
      ";

      if (!empty($search)) {
        $sql .= "
            WHERE CDSETOR LIKE :search
            OR UPPER(NMSETOR) LIKE :search
        ";
      }

      $stmt = $this->connection->prepare($sql);

      if (!empty($search)) {
        $searchTerm = '%' . strtoupper($search) . '%';
        $stmt->bindValue(':search', $searchTerm);
    }

    return $stmt->executeQuery()->fetchAllAssociative();
    }
  }
