<?php
namespace App\Service;

use App\Report\ReportBuilder;

class Report {
  private $reportFactory;
  private $util;
  private $orderService;

  public function __construct(\App\Factory\Report $reportFactory,
                              \App\Util\Util $util,
                              \App\Service\Order $orderService) {
      $this->reportFactory = $reportFactory;
      $this->util = $util;
      $this->orderService = $orderService;
  }

  public function generateReport($params) {
    $data = [];
    switch($params['report']) {
      case 'order':
        $order = $this->prepareOrderReport($params['paramsFilter']);
        $data = array_merge($order, $this->formatedData($params));
        return ReportBuilder::export($data);
      case 'production-order':
        $data = $this->prepareOrderProduction($params);
        $data = array_merge($data, $this->formatedData($params));
        return ReportBuilder::export($data);
        default:
          return null;
    }
  }

  private function prepareOrderReport($paramsFilter) {
    $order = $this->orderService->history($paramsFilter);
    return ['order' => $this->formatOrder($order)];
  }

  private function formatOrder($order) {
    $formattedOrders = array();
    foreach ($order as $items) {
        $formattedOrderItem = array(
          'IDSTCOMANDA'     => $this->util->formatStatus($items['IDSTCOMANDA'], $items['IDSINCMSDE']),
          'IDORGCMDVENDA'   => $this->util->commandSourceDictionary($items['IDORGCMDVENDA']),
          'IDSINCMSDE'      => $this->util->formatSynced($items['IDSINCMSDE']),
          'VRPEDIDO'        => $this->util->formatMoney($items['VRPEDIDO']),
          'VRTOTALPRODUTOS' => $this->util->formatMoney($items['VRTOTALPRODUTOS']),
          'VRDESCOMANDA'    => $this->util->formatMoney($items['VRDESCOMANDA']),
          'ACRESCIMO'       => $this->util->formatMoney($items['ACRESCIMO']),
          'VRACRCOMANDA'    => $this->util->formatMoney($items['VRACRCOMANDA']),
        );
      $formattedOrders[] = array_merge($items, $formattedOrderItem);
    }
    return $formattedOrders;
  }

  private function formatedData($params) {
    $data['title']  = isset($params['title']) ? $params['title'] : '';
    $data['report'] = isset($params['report']) ? $params['report'] : '';
    $data['format'] = isset($params['format']) ? $params['format'] : '';
    $data['visibleColumns'] = isset($params['visibleColumns']) ? $params['visibleColumns'] : '';
    $data['period'] = $params['dtInicial'] && $params['dtFinal'] ? $params['dtInicial'] . ' - ' . $params['dtFinal'] : '';
    return $data;
  }

  private function prepareOrderProduction($params = []) {
    $orderProduction = $this->reportFactory->orderProduction($params);
    if(empty($orderProduction)) throw new \Exception('Nenhum pedido encontrado.');

    return $this->groupOrdersByProduct($orderProduction);
  }

  private function groupOrdersByProduct($data) {
    $result = [];

    foreach ($data as $item) {
        $product = $item['PRODUTO'];
        $dataProduct = $item['DATA'];
        $key = $product . '_' . $dataProduct;

        $itemData = [
            'PEDIDO'        => $item['PEDIDO'],
            'IDORGCMDVENDA' => $this->util->commandSourceDictionary($item['IDORGCMDVENDA']),
            'ENDERECO'      => $item['ENDERECO'],
            'OBSERVACAO'    => isset($item['OBSERVACAO'])
              ? implode("\n", array_map('trim', explode(';', $item['OBSERVACAO'])))
              : '',
            'OBSERVACAOCMD' => isset($item['OBSERVACAOCMD'])
              ? implode("\n", array_map('trim', explode(';', $item['OBSERVACAOCMD'])))
              : '',
            'DATA'          => $item['DATA'],
            'HORARIO'       => $item['HORARIO'],
            'QUANTIDADE'    => $item['QUANTIDADE'],
            'CLIENTE'       => $item['CLIENTE']
        ];

        if (isset($result[$key])) {
            $result[$key]['ITENS'][] = $itemData;
        } else {
            $result[$key] = [
                'PRODUTO' => $product,
                'DATA'    => $dataProduct,
                'QTDE'    => 0,
                'ITENS'   => [$itemData]
            ];
        }

        $result[$key]['QTDE'] = count($result[$key]['ITENS']);
    }

    return [
        'SETOR'    => $data[0]['SETOR'],
        'PRODUTOS' => array_values($result),
    ];
  }

  public function getUnit($params) {
    return $this->reportFactory->getUnit($params);
  }

  public function getSector($params) {
    return $this->reportFactory->getSector($params);
  }
}