<?php
namespace App\Util;

class Util {
    private $connection;

    public function __construct (\Doctrine\DBAL\Connection $connection) {
        $this->connection = $connection;
    }
    
    public function newCode ($counterName, $size) {
        /* NOVOCODIGO */
        try {
            $nrseq = null;
            $connection = $this->connection;
            $storedProcedure = new StoredProcedure($connection, "NOVO_CODIGO_NT");
            $storedProcedure->addParam(new Param("P_CONTADOR", Param::PARAM_INPUT, $counterName, Param::PARAM_TYPE_STR, strlen($counterName)));
            $storedProcedure->addParam(new Param("P_SEQUENCIAL", Param::PARAM_OUTPUT, $nrseq, Param::PARAM_TYPE_INT, 20));
            $result = $storedProcedure->execute();
            return substr($result["P_SEQUENCIAL"], strlen($result["P_SEQUENCIAL"])-$size);
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage());
        }
    }
    
    /**
     *
     *
     * @param StoredProcedure $procedure The procedure to be executed.
     * @param array $paramValues Param values to override values present in Param instance.
     *
     * @return array The output parameters indexed by name.
     */
    public function executeProcedure(StoredProcedure $procedure, array $paramValues = array()) {
        $queryStmt = $this->getQueryString($procedure);
        $stmt = $this->connection->prepare($queryStmt);
        $valuesOutput = array();
        foreach ($procedure->getParams() as $param) {
            if ($param->isOutput()) {
                $valuesOutput[$param->getName()] = $param->getValue();
                $stmt->bindParam($param->getParamAlias(), $valuesOutput[$param->getName()], $param->getType(), $param->getLength());
            } else {
                $stmt->bindValue($param->getParamAlias(), $param->getValue(), $param->getType());
            }
        }

        $stmt->execute();
        return $valuesOutput;
    }

    /**
     * @param StoredProcedure $procedure
     * @return string
     */
    protected function getQueryString(StoredProcedure $procedure) {
        $paramNames = array_map(function (Param $param) {
            return ":" . $param->getName();
        }, $procedure->getParams());
        $paramString = implode(", ", $paramNames);
        return "CALL " . $procedure->getName() . " ( " . $paramString . " )";
    }

    public function inside($point, $vs) {
        $x = $point[0];
        $y = $point[1];

        $inside = false;
        for ($i = 0, $j = count($vs) - 1; $i < count($vs); $j = $i++) {
            $xi = (float)$vs[$i][0];
            $yi = (float)$vs[$i][1];
            $xj = (float)$vs[$j][0];
            $yj = (float)$vs[$j][1];

            $intersect = (($yi > $y) != ($yj > $y)) && ($x < ($xj - $xi) * ($y - $yi) / ($yj - $yi) + $xi);
            if ($intersect)
                $inside = !$inside;
        }
        return $inside;
    }

    public function removeAcentos($texto) {
        return preg_replace(array("/(á|à|ã|â|ä)/","/(Á|À|Ã|Â|Ä)/","/(é|è|ê|ë)/","/(É|È|Ê|Ë)/","/(í|ì|î|ï)/","/(Í|Ì|Î|Ï)/","/(ó|ò|õ|ô|ö)/","/(Ó|Ò|Õ|Ô|Ö)/","/(ú|ù|û|ü)/","/(Ú|Ù|Û|Ü)/","/(ñ)/","/(Ñ)/","/(ç)/","/(Ç)/"),explode(" ","a A e E i I o O u U n N c C"),$texto);
    }

    public function formatsCPF($cpf) {
		$cpf = str_replace('.', '', $cpf);
		$cpf = str_replace('-', '', $cpf);
		
		return $cpf;
	}

    public function getEnv($name) {
        return $_ENV[$name];
    }

    public static function encrypt($password) {
        return md5($password);
    }

    public static function getDelphiPasswordEncrypted($password, $isEncrypt = true) {
        $url = $_ENV['ASP_URL'] . '/comwebservice/service.asmx?wsdl';

        $options = array(
            'encoding'    => 'ISO-8859-1',
            'trace'       => true,
            'exceptions'  => true,
            'compression' => SOAP_COMPRESSION_ACCEPT | SOAP_COMPRESSION_GZIP          
        );

        $args = array(
            new \SoapVar($password, XSD_STRING, 'string', "http://www.w3.org/2001/XMLSchema")
        );
        
        $cryptType = $isEncrypt ? 'GetCriptoPass' : 'GetUnCriptoPass';
        $params    = array(
            'rdm'    => 'Bib_html.rdmDBGeralHtm',
            'method' => $cryptType,
            'args'   => $args,
        );

        try {
            $client     = new \SoapClient($url, $options);
            $result     = $client->executeCOMMethod($params);
            $encryption = $result->executeCOMMethodResult;
            return $encryption;
        } catch (\Exception $e) {     
            throw new \Exception($e->getMessage());
        }
    }

    public static function commandSourceDictionary($value) {
        $commandSourceDictionary = [
            'DLV_SIT' => 'Site/App (teknisa)',
            'DLV_CAL' => 'Callcenter',
            'DLV_INT' => 'Site/Integracao',
            'DLV_FOS' => 'ForSale',
            'DLV_FNC' => 'ForSale',
            'DLV_SAT' => 'ForSale',
            'DLV_IFO' => 'iFood',
            'DLV_RAP' => 'Rappi',
            'DLV_NMO' => 'Neemo',
            'DLV_ORD' => 'Order API',
            'DLV_DDI' => 'Delivery Direto',
            'DLV_PED' => 'Pede Pronto',
            'DLV_ANT' => 'Anota Ai',
            'DLV_SPB' => 'Shoppub'
        ];

        if (isset($commandSourceDictionary[$value])) {
            return $commandSourceDictionary[$value];
        }
        return 'Outros';
    }

    public function getCoordinatesByAddress($streetAddress, $number, $neighborhood, $cityName, $stateCode){
	    $address = $streetAddress.', '.$number;
        if(!empty($neighborhood)){
            $address   = $address.', '.$neighborhood;
        }
        if(!empty($cityName)){
            $address   = $address.', '.$cityName;
        }
        if(!empty($stateCode)){
            $address   = $address.', '.$stateCode;
        }
        $addressUrl = urlencode($address);
        $url        = $this->getEnv('GEOCODE_URL')."&address=".$addressUrl;
        $respJSON   = $this->_connectToWebService($url);
        $resp       = json_decode($respJSON, true);

        if($this->verifyLocationService($resp)) {
            $geometries = [];
            if(!empty($resp['results'])) {
                $resp['results'] = array_values(array_filter($resp['results'], function($result) use ($cityName, &$geometries) {
                    foreach ($result['address_components'] as $c) {
                        if($c['long_name'] == $cityName) {
                            return $result;
                        }
                    }
                    
                    if(!empty($result['geometry']['location'])) {
                        $url = $this->getEnv('GEOCODE_URL')."address=".urlencode($result['geometry']['location']['lat'].', '.$result['geometry']['location']['lng']);
                        array_push($geometries, $url);
                    }
                }));
                
                if(empty($resp['results']) && !empty($geometries)) {
                    $respLatLng = json_decode($this->_connectToWebService($geometries[0]), true);
                    
                    $resp['results'] = !empty($respLatLng['results'][0]) ? [$respLatLng['results'][0]] : [];
                }
                
            }
        }
        
        return $resp;
	}

    public function verifyLocationService($location) {
        $check = true;
        try {
            if(!empty($location) && isset($location['status']) && $location['status'] == 'OK' && !empty($location['results'])) {
                foreach ($location['results'] as $l) {
                    if(!isset($l['geometry']) && !isset($l['geometry']['location'])) {
                        $check = false;
                    }
                }
            } else $check = false;
            
            
            return $check;
        } catch(\Exception $e) {
            return false;
        }
    }

    public function _connectToWebService($url) {
        $curl = curl_init();

        curl_setopt($curl, CURLOPT_URL, $url);
        curl_setopt($curl, CURLOPT_CUSTOMREQUEST, 'GET');
        curl_setopt($curl, CURLOPT_HTTPHEADER, array('Content-Type: application/x-www-form-urlencoded'));
        curl_setopt($curl, CURLOPT_HTTP_VERSION, CURL_HTTP_VERSION_1_1);
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($curl, CURLOPT_TIMEOUT, 30);
        curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, false);

        /* Executes the webservice. */
        $curl_response = curl_exec($curl);
        
        curl_close($curl);
        return $curl_response;
    }

    public static function formatMoney($value) {
        $formatValue = number_format($value, 2, ',', '.');
        $formatValue = 'R$ ' . $formatValue;
        return $formatValue;
    }

    public static function formatSynced($value) {
        $formatSynced = [
            'N' => 'Não',
            'S' => 'Sim',
        ];

        return $formatSynced[$value];
    }

    public static function formatStatus($status, $isSincMSDE) {
        $commandStatusMap = [
            '1' => $isSincMSDE === 'N' ? '-' : 'Em produção',
            '2' => $isSincMSDE === 'N' ? '-' : 'Em produção',
            '3' => $isSincMSDE === 'N' ? '-' : 'Em produção',
            '4' => $isSincMSDE === 'N' || $isSincMSDE === 'S' ? 'Cancelado' : '',
            '5' => $isSincMSDE === 'N' ? '-' : 'Expedido',
            '6' => $isSincMSDE === 'N' ? '-' : 'Não pago',
            'X' => $isSincMSDE === 'N' ? '-' : 'Finalizado',
            'P' => $isSincMSDE === 'N' ? '-' : 'Impresso',
            'A' => $isSincMSDE === 'N' ? '-' : 'Agendado',
            'N' => $isSincMSDE === 'N' ? '-' : 'Não sincronizado',
        ];

        return $commandStatusMap[$status];
    }

    public function layoutDefault($idativo = 'S') {
        return array (
            'NMLAYOUT'   => 'Padrão',
            'DSCOLUNAS'  => array (
                'CDFILIAL','CDLOJA','NRCOMANDA','NRPEDCENTRAL','NRCOMANDAEXT','DTHRABERMESA','NMCONSUMIDOR','IDORGCMDVENDA','IDSTCOMANDA','TEMPOPEDIDO','VRPEDIDO','IDSINCMSDE','AGENDADO','TOGO','NRTELECONS', 'cancel-order', 'antecipation', 'QTDPROD', 'DSAREAATEND'
            ),
            'IDATIVO'    => $idativo,
         );
    }

    public function encodeBase64($textBase64) {
        return base64_encode($textBase64);
    }

    public function decodeBase64($textBase64) {
        return base64_decode($textBase64);
    }

	public function mapState($state){
		
		$state = $this->removeAcentos($state);
		
		$stateArr = array(
				'Acre'				  => 'AC',
				'Alagoas'             => 'AL',
				'Amapa'               => 'AP',
				'Amazonas'            => 'AM',
				'Bahia'               => 'BA',
				'Ceará'               => 'CE',
				'Distrito Federal'    => 'DF',
				'Espirito Santo'      => 'ES',
				'Goias'               => 'GO',
				'Maranhao'            => 'MA',
				'Mato Grosso'         => 'MT',
				'Mato Grosso do Sul'  => 'MS',
				'Minas Gerais'        => 'MG',
				'Para'                => 'PA',
				'Paraiba'             => 'PB',
				'Parana'              => 'PR',
				'Pernambuco'          => 'PE',
				'Piaui'               => 'PI',
				'Rio de Janeiro'      => 'RJ',
				'Rio Grande do Norte' => 'RN',
				'Rio Grande do Sul'   => 'RS',
				'Rondonia'            => 'RO',
				'Roraima'             => 'RR',
				'Santa Catarina'      => 'SC',
				'Sao Paulo'           => 'SP',
				'Sergipe'             => 'SE',
				'Tocantins'           => 'TO'
			);
			
		$stateCode = isset($stateArr[$state]) ? $stateArr[$state] : null; 
		
		return $stateCode;
	}

  public function floor($val, $precision = 2) {
      if ($precision < 0) { 
          $precision = 0; 
      }
      
      $numPointPosition = intval(strpos($val, '.'));
      
      if ($numPointPosition === 0) { //$val is an integer
          return $val;
      }

      return substr($val, 0, $numPointPosition + $precision + 1);
  }
}

